class PrincipalType:
    """Specifies the type of a principal."""

    def __init__(self):
        pass

    None_ = 0
    User = 1
    DistributionList = 2
    SecurityGroup = 4
    SharePointGroup = 8
    All = 15
