from office365.directory.directory_object import DirectoryObject


class GroupSettingTemplate(DirectoryObject):
    """Group setting templates represent system-defined settings available to the tenant. Group settings can be
    created based on the available groupSettingTemplates, and values changed from their preset defaults. """
