from office365.entity import Entity
from office365.sharepoint.base_entity import BaseEntity


class GetExternalUsersResults(BaseEntity):
    pass


class RemoveExternalUsersResults(BaseEntity):
    pass


class SPOUserSessionRevocationResult(Entity):
    pass
