class ChangeType:

    def __init__(self):
        pass

    NoChange = "NoChange"
    Add = "Add"
    Update = "Update"
    DeleteObject = "DeleteObject"
    Rename = "Rename"
    MoveAway = "MoveAway"
    MoveInto = "MoveInto"
    Restore = "Restore"
    RoleAdd = "RoleAdd"
    RoleDelete = "RoleDelete"
    RoleUpdate = "RoleUpdate"
    AssignmentAdd = "AssignmentAdd"
    AssignmentDelete = "AssignmentDelete"
    MemberAdd = "MemberAdd"
    MemberDelete = "MemberDelete"
    SystemUpdate = "SystemUpdate"
    Navigation = "Navigation"
    ScopeAdd = "ScopeAdd"
    ScopeDelete = "ScopeDelete"
    ListContentTypeAdd = "ListContentTypeAdd"
    ListContentTypeDelete = "ListContentTypeDelete"
