from office365.runtime.client_value import ClientValue


class MeetingTimeSuggestion(ClientValue):
    """
    A meeting suggestion that includes information like meeting time, attendance likelihood, individual availability,
    and available meeting locations.
    """
    pass
