from office365.directory.identities.identity_provider_base import IdentityProviderBase


class BuiltInIdentityProvider(IdentityProviderBase):
    """
    Represents built-in identity providers with External Identities for an Azure Active Directory tenant.

    """
    pass
