from office365.runtime.client_value import ClientValue


class GeoCoordinates(ClientValue):
    """
    The GeoCoordinates resource provides geographic coordinates and elevation of a location based on metadata
    contained within the file. If a DriveItem has a non-null location facet, the item represents a file with
    a known location associated with it.
    """
    pass
