from office365.runtime.client_value import ClientValue


class PersonOrGroupColumn(ClientValue):
    """The personOrGroupColumn on a columnDefinition resource indicates that the column's values represent
    a person or group chosen from the directory."""
    pass
