from office365.sharepoint.base_entity import BaseEntity


class Audit(BaseEntity):
    """
    Enables auditing of how site collections, sites, lists, folders, and list items are accessed, changed, and used.
    """
    pass
