from office365.runtime.client_value import ClientValue


class ContentTypeOrder(ClientValue):
    """The contentTypeOrder resource specifies in which order the Content Type will appear in the selection UI."""
    pass
