from office365.entity import Entity


class UserFlowLanguagePage(Entity):
    """
    Determines the user flow language pages that are shown to users during a user flow.
    These language pages include both the default language translations provided by Microsoft,
    or custom pages that can be created to customize the language translations.
    """
    pass
