from office365.runtime.client_value import ClientValue


class AssignedPlan(ClientValue):
    """
    The assignedPlans property of both the user entity and the organization entity is a collection of assignedPlan.
    """
    pass
