from office365.runtime.client_value import ClientValue


class CalculatedColumn(ClientValue):
    """The calculatedColumn on a columnDefinition resource indicates that the column's
    data is calculated based on other columns in the site."""
    pass
