from office365.runtime.client_value import ClientValue


class RecentNotebook(ClientValue):
    """A recently accessed OneNote notebook. A recentNotebook is similar to a notebook but has fewer properties."""
    pass
