from office365.runtime.client_value import ClientValue


class Thumbnail(ClientValue):
    """
    The thumbnail resource type represents a thumbnail for an image, video, document,
        or any item that has a bitmap representation.
    """
    pass
