from office365.entity import Entity


class TeamsTemplate(Entity):
    """A team template is a blueprint for creating a team in Microsoft Teams.
    A template specifies the structure, settings, and even content that should be provisioned in a new team created
    using the template. Microsoft provides a suite of base templates
    and customers can save their own custom templates."""
    pass
