class ViewType:
    """Specifies the type of the list view."""
    def __init__(self):
        pass

    NoneType = 0
    Html = 1
    Grid = 2048
    Calendar = 524288
    Recurrence = 8193
    Chart = 131072
    Gantt = 67108864
