from office365.entity import Entity


class GroupSetting(Entity):
    """Group settings control behaviors such as blocked word lists for group display names or whether guest users are
    allowed to be group owners. """
