from office365.runtime.client_value import ClientValue


class Audio(ClientValue):
    """
    The Audio resource groups audio-related properties on an item into a single structure.

    If a DriveItem has a non-null audio facet, the item represents an audio file.
    The properties of the Audio resource are populated by extracting metadata from the file.
    """
    pass
