from office365.runtime.client_value import ClientValue


class Photo(ClientValue):
    """The photo resource provides photo and camera properties, for example, EXIF metadata, on a driveItem."""
    pass
