from office365.entity import Entity


class ConditionalAccessPolicy(Entity):
    """
    Represents an Azure Active Directory conditional access policy.
    Conditional access policies are custom rules that define an access scenario.
    """
    pass
